// ----------------------------------------------------------------
// The contents of this file are distributed under the CC0 license.
// See http://creativecommons.org/publicdomain/zero/1.0/
// ----------------------------------------------------------------

#include "Controller.h"
#include "View.h"
#include <iostream>

using std::cout;

Controller::Controller(Model& model, View& view, bool* pKeepRunning, double trans_noise_dev)
: m_model(model), m_view(view), m_pKeepRunning(pKeepRunning), m_spaceDown(false), m_in(6), m_trans_dev(trans_noise_dev)
{
	m_in.fill(0.0);
}

Controller::~Controller()
{
}

void Controller::transition(size_t action)
{
	m_in[0] = m_model.state[0];
	m_in[1] = m_model.state[1];
	m_in[2 + action] = 1.0;
	const Vec& pred = m_model.trans.forwardprop(m_in);
	m_in[2 + action] = 0.0;
	m_model.state[0] += pred[0] + m_model.rand.normal() * m_trans_dev;
	m_model.state[1] += pred[1] + m_model.rand.normal() * m_trans_dev;
}

void Controller::update()
{
	SDL_Event event;
	while(SDL_PollEvent(&event))
	{
		if(event.type == SDL_QUIT)
			*m_pKeepRunning = false;
	}
	const Uint8* keys = SDL_GetKeyboardState(NULL);
	if(keys[SDL_SCANCODE_ESCAPE])
		*m_pKeepRunning = false;
	int mouseX, mouseY;
	Uint32 mouse_buttons = SDL_GetMouseState(&mouseX, &mouseY);
	if(keys[SDL_SCANCODE_SPACE])
	{
		if(!m_spaceDown)
		{
			cout << "Congratulations, you discovered what happens when you press the space bar!\n";
			cout.flush();
			m_spaceDown = true;
		}
	}
	else
		m_spaceDown = false;
	if(keys[SDL_SCANCODE_RIGHT])
		transition(1);
	if(keys[SDL_SCANCODE_LEFT])
		transition(0);
	if(keys[SDL_SCANCODE_UP])
		transition(2);
	if(keys[SDL_SCANCODE_DOWN])
		transition(3);
	if(mouse_buttons & SDL_BUTTON(SDL_BUTTON_LEFT))
	{
		//m_model.onClick(mouseX, mouseY);
	}
}


